﻿Imports System.Math
Imports System.Drawing.Drawing2D

Public Class Form1
    ' The given circles.
    Private GivenCircles As New List(Of Circle)()

    ' The solutions.
    Private SolutionCircles() As Circle

    ' CheckBoxes that control which circles are displayed.
    Private CheckBoxes() As CheckBox

    Private SolutionPens() As Pen = _
    { _
        Pens.Red, Pens.Green, Pens.Blue, Pens.Orange, _
        Pens.Lime, Pens.DeepSkyBlue, Pens.Pink, Pens.Purple _
    }

    ' Solve an example problem and display the result.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        CheckBoxes = New CheckBox() {chk1, chk2, chk3, chk4, chk5, chk6, chk7, chk8}
        DoubleBuffered = True

        SolutionCircles = New Circle() {}
    End Sub

    ' Draw the circles.
    Private Sub Form1_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles Me.Paint
        e.Graphics.Clear(BackColor)
        e.Graphics.SmoothingMode = SmoothingMode.AntiAlias

        ' Draw the solution circles.
        For i As Integer = 0 To SolutionCircles.Length - 1
            If (CheckBoxes(i).Checked) Then
                Dim solution_circle As Circle = SolutionCircles(i)

                If (chkFilled.Checked) Then
                    Dim clr As Color = Color.FromArgb(128, _
                        SolutionPens(i).Color.R, _
                        SolutionPens(i).Color.G, _
                        SolutionPens(i).Color.B)
                    Using circle_brush As New SolidBrush(clr)
                        solution_circle.Draw(e.Graphics, circle_brush)
                    End Using
                End If

                solution_circle.Draw(e.Graphics, SolutionPens(i))
            End If
        Next i

        ' Draw the given circles.
        For Each given_circle As Circle In GivenCircles
            Dim clr As Color = Color.FromArgb(128, 0, 0, 0)
            Using brush As New SolidBrush(clr)
                given_circle.Draw(e.Graphics, brush)
            End Using
            given_circle.Draw(e.Graphics, Pens.Black)
        Next given_circle

        ' Draw the enw circle if any.
        If (NewCircle IsNot Nothing) Then
            Using dashed_pen As New Pen(Color.Red)
                dashed_pen.DashStyle = DashStyle.Dash
                NewCircle.Draw(e.Graphics, dashed_pen)
            End Using
        End If
    End Sub

    ' Find the circles that touch each of the three input circles.
    Private Function FindApollonianCircles(ByVal given_circles As List(Of Circle)) As Circle()
        ' Make a list for results.
        Dim solution_circles As New List(Of Circle)()

        ' Loop over all of the signs.
        For Each s0 As Integer In New Integer() {-1, 1}
            For Each s1 As Integer In New Integer() {-1, 1}
                For Each s2 As Integer In New Integer() {-1, 1}
                    Dim new_circle As Circle = FindApollonianCircle( _
                        given_circles(0), given_circles(1), given_circles(2), _
                        s0, s1, s2)
                    If (new_circle IsNot Nothing) Then solution_circles.Add(new_circle)
                Next s2
            Next s1
        Next s0

        ' Return the results.
        Return solution_circles.ToArray()
    End Function

    ' Find a solution to Apollonius' problem.
    ' For discussion and method, see:
    '    http:'mathworld.wolfram.com/ApolloniusProblem.html
    '    http:'en.wikipedia.org/wiki/Problem_of_Apollonius#Algebraic_solutions
    ' For most of a Java code implementation, see:
    '    http:'www.diku.dk/hjemmesider/ansatte/rfonseca/implementations/apollonius.html
    Private Function FindApollonianCircle(ByVal c1 As Circle, ByVal c2 As Circle, ByVal c3 As Circle, ByVal s1 As Integer, ByVal s2 As Integer, ByVal s3 As Integer) As Circle
        ' Make sure c2 doesn't have the same X or Y coordinate as the others.
        Const tiny As Single = 0.0001F
        If ((Abs(c2.Center.X - c1.Center.X) < tiny) OrElse _
            (Abs(c2.Center.Y - c1.Center.Y) < tiny)) _
        Then
            Dim temp_circle As Circle = c2
            c2 = c3
            c3 = temp_circle
            Dim temp_s As Integer = s2
            s2 = s3
            s3 = temp_s
        End If
        If ((Abs(c2.Center.X - c3.Center.X) < tiny) OrElse _
            (Abs(c2.Center.Y - c3.Center.Y) < tiny)) _
        Then
            Dim temp_circle As Circle = c2
            c2 = c1
            c1 = temp_circle
            Dim temp_s As Integer = s2
            s2 = s1
            s1 = temp_s
        End If
        Debug.Assert( _
            (c2.Center.X <> c1.Center.X) AndAlso (c2.Center.Y <> c1.Center.Y) AndAlso _
            (c2.Center.X <> c3.Center.X) AndAlso (c2.Center.Y <> c3.Center.Y), _
            "Cannot find points without matching coordinates.")

        Dim x1 As Single = c1.Center.X
        Dim y1 As Single = c1.Center.Y
        Dim r1 As Single = c1.Radius
        Dim x2 As Single = c2.Center.X
        Dim y2 As Single = c2.Center.Y
        Dim r2 As Single = c2.Radius
        Dim x3 As Single = c3.Center.X
        Dim y3 As Single = c3.Center.Y
        Dim r3 As Single = c3.Radius

        Dim v11 As Single = 2 * x2 - 2 * x1
        Dim v12 As Single = 2 * y2 - 2 * y1
        Dim v13 As Single = x1 * x1 - x2 * x2 + y1 * y1 - y2 * y2 - r1 * r1 + r2 * r2
        Dim v14 As Single = 2 * s2 * r2 - 2 * s1 * r1

        Dim v21 As Single = 2 * x3 - 2 * x2
        Dim v22 As Single = 2 * y3 - 2 * y2
        Dim v23 As Single = x2 * x2 - x3 * x3 + y2 * y2 - y3 * y3 - r2 * r2 + r3 * r3
        Dim v24 As Single = 2 * s3 * r3 - 2 * s2 * r2

        Dim w12 As Single = v12 / v11
        Dim w13 As Single = v13 / v11
        Dim w14 As Single = v14 / v11

        Dim w22 As Single = v22 / v21 - w12
        Dim w23 As Single = v23 / v21 - w13
        Dim w24 As Single = v24 / v21 - w14

        Dim P As Single = -w23 / w22
        Dim Q As Single = w24 / w22
        Dim M As Single = -w12 * P - w13
        Dim N As Single = w14 - w12 * Q

        Dim a As Single = N * N + Q * Q - 1
        Dim b As Single = 2 * M * N - 2 * N * x1 + 2 * P * Q - 2 * Q * y1 + 2 * s1 * r1
        Dim c As Single = x1 * x1 + M * M - 2 * M * x1 + P * P + y1 * y1 - 2 * P * y1 - r1 * r1

        ' Find roots of a quadratic equation
        Dim solutions() As Double = QuadraticSolutions(a, b, c)
        If (solutions.Length < 1) Then Return Nothing
        Dim rs As Single = CSng(solutions(0))
        Dim xs As Single = M + N * rs
        Dim ys As Single = P + Q * rs

        Return New Circle(xs, ys, rs)
    End Function

    ' Return solutions to a quadratic equation.
    Private Function QuadraticSolutions(ByVal a As Double, ByVal b As Double, ByVal c As Double) As Double()
        Const tiny As Double = 0.000001
        Dim discriminant As Double = b * b - 4 * a * c

        ' See if there are no real solutions.
        If (discriminant < 0) Then Return New Double() {}

        ' See if there is one solution.
        If (discriminant < tiny) Then Return New Double() {-b / (2 * a)}

        ' There are two solutions.
        Return New Double() _
            { _
                (-b + Math.Sqrt(discriminant)) / (2 * a), _
                (-b - Math.Sqrt(discriminant)) / (2 * a) _
            }
    End Function

    ' Refresh.
    Private Sub chk_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chk1.CheckedChanged, chkFilled.CheckedChanged, chk8.CheckedChanged, chk7.CheckedChanged, chk6.CheckedChanged, chk5.CheckedChanged, chk4.CheckedChanged, chk3.CheckedChanged, chk2.CheckedChanged
        If (Not IgnoreRefresh) Then Refresh()
    End Sub

    ' Check or uncheck all CheckBoxes.
    Private IgnoreRefresh As Boolean = False
    Private Sub btnAllOrNone_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnAll.Click, btnNone.Click
        IgnoreRefresh = True

        Dim btn As Button = DirectCast(sender, Button)
        Dim is_checked As Boolean = (btn.Text = "All")
        For Each chk As CheckBox In CheckBoxes
            chk.Checked = is_checked
        Next chk

        IgnoreRefresh = False
        Refresh()
    End Sub

    ' Let the user draw a circle.
    Private NewCircle As Circle = Nothing
    Private Sub Form1_MouseDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseDown
        If (GivenCircles.Count = 3) Then GivenCircles = New List(Of Circle)()
        SolutionCircles = New Circle() {}
        NewCircle = New Circle(e.X, e.Y, 1)
        Refresh()
    End Sub
    Private Sub Form1_MouseMove(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseMove
        If (NewCircle Is Nothing) Then Return

        Dim dx As Single = e.Location.X - NewCircle.Center.X
        Dim dy As Single = e.Location.Y - NewCircle.Center.Y
        NewCircle.Radius = CSng(Math.Sqrt(dx * dx + dy * dy))
        Refresh()
    End Sub
    Private Sub Form1_MouseUp(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseUp
        If (NewCircle Is Nothing) Then Return

        GivenCircles.Add(NewCircle)
        NewCircle = Nothing
        If (GivenCircles.Count = 3) Then SolutionCircles = FindApollonianCircles(GivenCircles)
        Refresh()
    End Sub
End Class
